"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var IModuleBuilder_1 = require("./IModuleBuilder");
var core_1 = require("@ts-ioc/core");
var BootstrapModule_1 = require("./BootstrapModule");
/**
 * application builder.
 *
 * @export
 * @class ApplicationBuilder
 * @extends {ModuleBuilder<T>}
 * @template T
 */
var ApplicationBuilder = /** @class */ (function () {
    function ApplicationBuilder(baseURL) {
        this.baseURL = baseURL;
        this.usedModules = [];
        this.customRegs = [];
    }
    /**
     * get container
     *
     * @returns
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.getContainer = function () {
        if (!this.container) {
            this.container = this.getContainerBuilder().create();
        }
        return this.container;
    };
    /**
     * set container.
     *
     * @param {IContainer} container
     * @returns
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.setContainer = function (container) {
        if (container) {
            this.container = container;
            this.builder = container.get(core_1.ContainerBuilderToken);
        }
        return this;
    };
    /**
     * get container builder.
     *
     * @returns
     * @memberof ModuleBuilder
     */
    ApplicationBuilder.prototype.getContainerBuilder = function () {
        if (!this.builder) {
            this.builder = this.createContainerBuilder();
        }
        return this.builder;
    };
    /**
     * use container builder
     *
     * @param {IContainerBuilder} builder
     * @returns
     * @memberof ModuleBuilder
     */
    ApplicationBuilder.prototype.setContainerBuilder = function (builder) {
        this.builder = builder;
        this.container = null;
        return this;
    };
    /**
     * get module builer.
     *
     * @returns {IModuleBuilder<T>}
     * @memberof IApplicationBuilder
     */
    ApplicationBuilder.prototype.getModuleBuilder = function () {
        if (!this.moduleBuilder) {
            this.moduleBuilder = this.createModuleBuilder();
        }
        return this.moduleBuilder;
    };
    /**
     * set module builder.
     *
     * @param {IModuleBuilder<T>} builder
     * @returns {this}
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.setModuleBuilder = function (builder) {
        this.moduleBuilder = builder;
        return this;
    };
    /**
     * use configuration.
     *
     * @param {(string | AppConfiguration<T>)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    ApplicationBuilder.prototype.useConfiguration = function (config) {
        if (!this.globalConfig) {
            this.globalConfig = Promise.resolve(this.getDefaultConfig());
        }
        var pcfg;
        var builder = this.getContainerBuilder();
        if (core_1.isString(config)) {
            pcfg = builder.loader.load(config)
                .then(function (rs) {
                return rs.length ? rs[0] : null;
            });
        }
        else if (config) {
            pcfg = Promise.resolve(config);
        }
        if (pcfg) {
            this.globalConfig = this.globalConfig
                .then(function (cfg) {
                return pcfg.then(function (rcfg) {
                    var excfg = (rcfg['default'] ? rcfg['default'] : rcfg);
                    cfg = core_1.lang.assign(cfg || {}, excfg || {});
                    return cfg;
                });
            });
        }
        return this;
    };
    /**
     * use module, custom module.
     *
     * @param {...(LoadType | CustomRegister<T>)[]} modules
     * @returns {this}
     * @memberof PlatformServer
     */
    ApplicationBuilder.prototype.use = function () {
        var modules = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules[_i] = arguments[_i];
        }
        this.usedModules = this.usedModules.concat(modules);
        return this;
    };
    /**
     * register modules via custom.
     *
     * @param {...CustomRegister<T>[]} moduleRegs
     * @returns {this}
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.registerModules = function () {
        var moduleRegs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            moduleRegs[_i] = arguments[_i];
        }
        this.customRegs = this.customRegs.concat(moduleRegs);
        return this;
    };
    /**
     * build and bootstrap application.
     *
     * @param {(Token<T> | Type<any> | AppConfiguration<T>)} token
     * @returns {Promise<T>}
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.bootstrap = function (token) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.build(token)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * build application.
     *
     * @param {IModuleBuilder<T>} builder
     * @param {(Token<T> | Type<any> | AppConfiguration<T>)} token
     * @returns {Promise<any>}
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.build = function (token) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var container, builder, cfg, app;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        container = this.getContainer();
                        return [4 /*yield*/, this.registerExts(container)];
                    case 1:
                        _a.sent();
                        builder = this.getModuleBuilder();
                        return [4 /*yield*/, this.mergeConfigure(builder.getConfigure(token))];
                    case 2:
                        cfg = _a.sent();
                        this.bindConfiguration(container, cfg);
                        return [4 /*yield*/, this.initContainer(cfg, container)];
                    case 3:
                        _a.sent();
                        if (!cfg.bootstrap) {
                            cfg.bootstrap = (core_1.isToken(token) ? token : null);
                        }
                        return [4 /*yield*/, this.createInstance(builder, cfg)];
                    case 4:
                        app = _a.sent();
                        return [2 /*return*/, app];
                }
            });
        });
    };
    ApplicationBuilder.prototype.createInstance = function (builder, config) {
        return builder.build(config);
    };
    /**
     * create default module builder.
     *
     * @protected
     * @returns
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.createModuleBuilder = function () {
        return this.getContainer().get(IModuleBuilder_1.ModuleBuilderToken);
    };
    /**
     * create default container builder.
     *
     * @protected
     * @returns
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.createContainerBuilder = function () {
        return new core_1.DefaultContainerBuilder();
    };
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @returns {Promise<IContainer>}
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.registerExts = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var usedModules;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!container.has(BootstrapModule_1.BootstrapModule)) {
                            container.register(BootstrapModule_1.BootstrapModule);
                        }
                        if (!this.usedModules.length) return [3 /*break*/, 2];
                        usedModules = this.usedModules;
                        this.usedModules = [];
                        return [4 /*yield*/, container.loadModule.apply(container, [container].concat(usedModules))];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/, container];
                }
            });
        });
    };
    ApplicationBuilder.prototype.initContainer = function (config, container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var customs;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.customRegs.length) return [3 /*break*/, 2];
                        customs = this.customRegs;
                        this.customRegs = [];
                        return [4 /*yield*/, Promise.all(customs.map(function (cs) {
                                return cs(container, config, _this);
                            }))];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/, container];
                }
            });
        });
    };
    ApplicationBuilder.prototype.bindConfiguration = function (container, config) {
        if (this.baseURL) {
            config.baseURL = this.baseURL;
        }
    };
    /**
     * meger config configuration with global config.
     *
     * @protected
     * @param {AppConfiguration<T>} cfg
     * @returns {Promise<AppConfiguration<T>>}
     * @memberof ApplicationBuilder
     */
    ApplicationBuilder.prototype.mergeConfigure = function (cfg) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var gcfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.globalConfig) {
                            this.useConfiguration();
                        }
                        return [4 /*yield*/, this.globalConfig];
                    case 1:
                        gcfg = _a.sent();
                        return [2 /*return*/, core_1.lang.assign({}, gcfg, cfg)];
                }
            });
        });
    };
    ApplicationBuilder.prototype.getDefaultConfig = function () {
        return { debug: false };
    };
    ApplicationBuilder.classAnnations = { "name": "ApplicationBuilder", "params": { "constructor": ["baseURL"], "getContainer": [], "setContainer": ["container"], "getContainerBuilder": [], "setContainerBuilder": ["builder"], "getModuleBuilder": [], "setModuleBuilder": ["builder"], "useConfiguration": ["config"], "use": ["modules"], "registerModules": ["moduleRegs"], "bootstrap": ["token"], "build": ["token"], "createInstance": ["builder", "config"], "createModuleBuilder": [], "createContainerBuilder": [], "registerExts": ["container"], "initContainer": ["config", "container"], "bindConfiguration": ["container", "config"], "mergeConfigure": ["cfg"], "getDefaultConfig": [] } };
    return ApplicationBuilder;
}());
exports.ApplicationBuilder = ApplicationBuilder;

//# sourceMappingURL=sourcemaps/ApplicationBuilder.js.map
